import argumentation.*;

import java.io.File;
import java.io.FileReader;
import java.io.StringReader;
import java.io.IOException;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.text.html.HTMLDocument;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
//import javax.swing.event.ListSelectionListener;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.JEditorPane;
import javax.swing.JTabbedPane;

import java.awt.*;
import java.awt.event.*;
import javax.swing.KeyStroke;

import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JFrame;


/**
This class represent the layout for the discussion game.
*/
public class DiscussionGameLayout extends DiscussionGameOutput implements ActionListener //, ListSelectionListener
{
	private DiscussionGameHandler dgh;

	String output = "";
	boolean playing;

	//FRAMES
	private JFrame frame;
	private JFrame argumentsFrame;
	private ExtensionLayout extensionsFrame;
	private JFrame helpFrame;

	//MAIN FRAME
	private JMenuBar menuBar;			//menu
		private JMenu fwMenu;
			private JMenuItem menuLoad;
			private JMenuItem menuReload;
			private JMenuItem menuSwitch;
			private JMenuItem menuExit;
		private JMenu extMenu;
			private JMenuItem menuExts;
		private JMenu hlpMenu;
			private JMenuItem menuWHap;
			private JMenuItem menuHelp;
			private JMenuItem menuAbout;

	private JFileChooser fc;			//file chooser
	private JEditorPane outputView;		//output area

	//ARGUMENTS FRAME
	JList argList;						//arguments list
	JButton selectButton;				//question/discuss button

	/**
	Constructs a new <CODE>DiscussionGameLayout</CODE> with the given <CODE>DiscussionGameHandler</CODE>.
	*/
	public DiscussionGameLayout(DiscussionGameHandler d)
	{
		dgh = d;

		//MAIN FRAME
		frame = new JFrame("Discussion Game");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLocation(200, 200);

		//Create and set up the content pane.
		frame.setJMenuBar(createMenuBar());
		frame.setContentPane(createContentPane());

		//Display the window.
		frame.setSize(700, 500);
		frame.setVisible(true);

		//Set the file chooser
		fc = new JFileChooser();
		fc.setCurrentDirectory(new File(System.getProperty("user.dir")));

		//ARGUMENTS FRAME
		argumentsFrame = new JFrame("Arguments & Input Manager");
		argumentsFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		argumentsFrame.setVisible(false);
		//argumentsFrame.setAlwaysOnTop(true);
		//argumentsFrame.setUndecorated(true);
		argumentsFrame.setSize(300, 200);
		argumentsFrame.setContentPane(createArgContentPane());
		argumentsFrame.setLocation(frame.getLocation().x + 600, frame.getLocation().y + 300);


		//EXTENSIONS FRAME
		extensionsFrame = new ExtensionLayout("Extensions Manager", dgh);
		extensionsFrame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		extensionsFrame.setVisible(false);
		extensionsFrame.setSize(400, 300);
		extensionsFrame.setLocation(frame.getLocation().x + 600, frame.getLocation().y);

		//HELP FRAME
		helpFrame = new JFrame("Help");
		helpFrame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		helpFrame.setVisible(false);
		helpFrame.setSize(500, 500);
		helpFrame.setContentPane(createHelpContentPane());
		helpFrame.setLocation(frame.getLocation().x + 600, frame.getLocation().y - 50);
	}

	public static void main(String[] args)
	{
		DiscussionGameHandler h = new DiscussionGameHandler();
		MessageGenerator mg = new DefaultMessageGenerator(h.getArgs());
		h.setMessageGenerator(mg);
		DiscussionGameLayout l = new DiscussionGameLayout(h);
		h.setOutput(l);
	}


    public JPanel createContentPane()
    {
        //Create the content-pane-to-be.
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.setOpaque(true);

        //Create the output area.
        outputView = createEditorPane();
        JScrollPane scrollPane = new JScrollPane(outputView);

        //Add the text area to the content pane.
        contentPane.add(scrollPane, BorderLayout.CENTER);

        return contentPane;
    }


    private JEditorPane createEditorPane()
    {
		String st = "<HTML><BODY style=\"background-color:#ffffff\";><CENTER><BR><BR>" +
					"<P style=\"background-color:#feffb8;font:verdana;font-weight:bold\">University of Luxembourg</P>" +
					"<P style=\"background-color:#eeeeee;font:verdana;font-size:20\">Argumentation game player</P>" +
					"<P style=\"background-color:#feffb8;font:verdana\">12/01/2009</P>" +
					"</CENTER></BODY></HTML>";
        JEditorPane jep = new JEditorPane();
        jep.setEditable(false);
		jep.setContentType("text/html");
		jep.setText(st);

        return jep;
    }


    private JMenuBar createMenuBar()
    {
        //Create the menu bar.
        menuBar = new JMenuBar();

        //Build the framework menu.
        fwMenu = new JMenu("File");
        fwMenu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(fwMenu);
	        //Items
	        menuLoad = new JMenuItem("Load framework...", KeyEvent.VK_L);
	        menuLoad.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
	    	menuLoad.addActionListener(this);
        	fwMenu.add(menuLoad);

	        menuReload = new JMenuItem("Reload framework", KeyEvent.VK_R);
	        menuReload.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
	    	menuReload.addActionListener(this);
	    	menuReload.setEnabled(false);
        	fwMenu.add(menuReload);

	        menuSwitch = new JMenuItem("Switch language generation...", KeyEvent.VK_L);
	        menuSwitch.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
	    	menuSwitch.addActionListener(this);
	    	menuSwitch.setEnabled(true);
        	fwMenu.add(menuSwitch);

	        menuExit = new JMenuItem("Exit", KeyEvent.VK_E);
	        menuExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.ALT_MASK));
	    	menuExit.addActionListener(this);
        	fwMenu.add(menuExit);

        //Build the extensions menu.
        extMenu = new JMenu("Extensions");
        extMenu.setMnemonic(KeyEvent.VK_E);
        menuBar.add(extMenu);
	        //Items
	        menuExts = new JMenuItem("Show extensions...", KeyEvent.VK_G);
	        menuExts.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
	    	menuExts.addActionListener(this);
	    	menuExts.setEnabled(false);
        	extMenu.add(menuExts);

        //Build the help menu.
        hlpMenu = new JMenu("Help");
        hlpMenu.setMnemonic(KeyEvent.VK_L);
        menuBar.add(hlpMenu);
	        //Items
	        menuWHap = new JMenuItem("What's happening??", KeyEvent.VK_W);
	        menuWHap.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
	    	menuWHap.addActionListener(this);
	    	menuWHap.setEnabled(false);
        	hlpMenu.add(menuWHap);

	        menuHelp = new JMenuItem("Help...", KeyEvent.VK_H);
	        menuHelp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
	    	menuHelp.addActionListener(this);
        	hlpMenu.add(menuHelp);

	        menuAbout = new JMenuItem("About", KeyEvent.VK_A);
	        //menuAbt.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
	    	menuAbout.addActionListener(this);
        	hlpMenu.add(menuAbout);

        return menuBar;
    }


	private JPanel createArgContentPane()
	{
        //Create the content-pane-to-be.
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.setOpaque(true);

        //Create the list.
        argList = new JList();
        argList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        //argList.addListSelectionListener(this);
        JScrollPane scrollPane = new JScrollPane(argList);

        //Create the button
        selectButton = new JButton("Question");
        selectButton.addActionListener(this);
        selectButton.setEnabled(false);

        //Add the stuff to the content pane.
        contentPane.add(scrollPane, BorderLayout.CENTER);
        contentPane.add(selectButton, BorderLayout.SOUTH);

        return contentPane;
	}

	private JPanel createHelpContentPane()
	{
        //Create the content-pane-to-be.
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.setOpaque(true);

        JEditorPane hep = new JEditorPane();
        hep.setEditable(false);
		hep.setContentType("text/html");
		hep.setText("<html><head><title>An Implementation of Argument Based Discussion</title> </head> <body lang=IT style='tab-interval:35.4pt'> <div class=Section1><pre><span lang=EN-GB style='font-size:14.0pt;font-family: Verdana;mso-ansi-language:EN-GB'><BR><BOLD>An Implementation of Argument Based Discussion</BOLD></span></pre> <p class=MsoNormal><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'><o:p>&nbsp;</o:p></span></p> <p class=MsoNormal><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'>Those are very basic instruction for the use of the program.<o:p></o:p></span></p> <p class=MsoNormal><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'>In order to start a game:<o:p></o:p></span></p> <ol style='margin-top:0cm' start=1 type=1> <li class=MsoNormal style='mso-list:l0 level1 lfo1;tab-stops:list 36.0pt'><span class=GramE><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'>open</span></span><span lang=EN-GB style='font-family:Verdana; mso-ansi-language:EN-GB'> an argumentation framework. You can find a couple of examples in the program folder. In order to load one, just use the menu File </span><span lang=EN-US style='font-family:Wingdings; mso-ascii-font-family:Verdana;mso-hansi-font-family:Verdana;mso-ansi-language: EN-US;mso-char-type:symbol;mso-symbol-font-family:Wingdings'><span style='mso-char-type:symbol;mso-symbol-font-family:Wingdings'>--></span></span><span lang=EN-US style='font-family:Verdana;mso-ansi-language:EN-US'> </span><span lang=EN-GB style='font-family:Verdana;mso-ansi-language:EN-GB'>Load and browse for the correct file;<o:p></o:p></span></li> <li class=MsoNormal style='mso-list:l0 level1 lfo1;tab-stops:list 36.0pt'><span class=GramE><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'>once</span></span><span lang=EN-GB style='font-family:Verdana; mso-ansi-language:EN-GB'> loaded a framework, you should see the list of arguments of the framework in the <span class=SpellE>Arguments&amp;Input</span> frame. Choose an argument and question it;<o:p></o:p></span></li> <li class=MsoNormal style='mso-list:l0 level1 lfo1;tab-stops:list 36.0pt'><span class=GramE><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'>once</span></span><span lang=EN-GB style='font-family:Verdana; mso-ansi-language:EN-GB'> questioned an argument, you can choose whether accept the applications opinion or discuss it. If you start discussing, you can choose next argument to use in the discussion from the <span class=SpellE>Arguments&amp;Input</span> frame, selecting it and pressing the Discuss button;<o:p></o:p></span></li> <li class=MsoNormal style='mso-list:l0 level1 lfo1;tab-stops:list 36.0pt'><span lang=EN-GB style='font-family:Verdana;mso-ansi-language:EN-GB'>if you need to view the loaded framework or its extensions, just use the menu Framework or Extensions;<o:p></o:p></span></li> <li class=MsoNormal style='mso-list:l0 level1 lfo1;tab-stops:list 36.0pt'><span class=GramE><span lang=EN-GB style='font-family:Verdana;mso-ansi-language: EN-GB'>the</span></span><span lang=EN-GB style='font-family:Verdana; mso-ansi-language:EN-GB'> menu Help offers this introduction and the Whats happening? dialogue, which explains, during a game, what type of game is going on and why.<o:p></o:p></span></li> </ol> </div> </body> </html>");

		contentPane.add(new JScrollPane(hep));

        return contentPane;
	}



	//ACTION MANAGEMENT

    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();

        if (source == menuLoad)
			loadFramework();
		else if (source == menuReload)
			reloadFramework();
		else if (source == menuSwitch)
			switchMessageGenerator();
		else if (source == menuExit)
        	System.exit(0);
		else if (source == menuExts)
			showExtensions();
		else if (source == menuWHap)
			showWhatHappens();
		else if (source == menuHelp)
			showHelp();
		else if (source == menuAbout)
			showAbout();
		else if (source == selectButton)
		{
			if (playing)
			{
				String[] options = dgh.discuss(argList.getLeadSelectionIndex());

				if (options == null)
				{
					selectButton.setText("Question");
					updateArgList(dgh.getMessageGenerator().getArgs());
					playing = false;
					menuWHap.setEnabled(false);
				}
				else
				{
					updateArgList(options);
				}
			}
			else
			{
				String[] options = dgh.question(argList.getLeadSelectionIndex());
				int ans = JOptionPane.showOptionDialog(frame,
														"What do you think?",
														"We can discuss...",
														JOptionPane.YES_NO_OPTION,
														JOptionPane.QUESTION_MESSAGE,
														null,     		//Icon
														options,  		//button names
														options[1]);	//default button
				if (ans != 0)
				{
					selectButton.setText("Discuss");
					String[] nextArgs = dgh.startDiscussion(ans);
					updateArgList(nextArgs);
					playing = true;
					menuWHap.setEnabled(true);
				}
			}
		}
    }

    private void loadFramework()
    {
		int returnVal = fc.showOpenDialog(null);

		if (returnVal == JFileChooser.APPROVE_OPTION)
			reloadFramework();
	}

    private void reloadFramework()
    {
		File file = fc.getSelectedFile();

		try
		{
			dgh.load(file.getPath());
			updateArgList(dgh.getMessageGenerator().getArgs());
			extensionsFrame.updateContent();

			menuReload.setEnabled(true);
			menuExts.setEnabled(true);
			argumentsFrame.setVisible(true);
			outputView.setText("");

			selectButton.setEnabled(true);
			selectButton.setText("Question");
			playing = false;
		}
		catch (RuntimeException exc)
		{
			System.out.println(exc);
			Object[] options = {"Ok", "More..."};
			int n = JOptionPane.showOptionDialog(frame,
							"The selected file is not correctly formatted.",
							"File loading error",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.ERROR_MESSAGE,
							null,        //icon
							options,     //buttons
							options[1]); //default button
			if (n == 1)
				JOptionPane.showMessageDialog(frame, exc.getMessage(), "File loading error details", JOptionPane.WARNING_MESSAGE);
		}
	}

	private void switchMessageGenerator()
	{
		int n = JOptionPane.showConfirmDialog(
				frame,
				"Do you want to change language generation mode?",
				"Language Generation",
				JOptionPane.YES_NO_OPTION);

		if (n == JOptionPane.NO_OPTION)
			return;

		if (dgh.getMessageGenerator() instanceof DefaultMessageGenerator)
		{
			dgh.setMessageGenerator(new NaturalLanguageMessageGenerator(dgh.getArgs()));
		}
		else
		{
			dgh.setMessageGenerator(new DefaultMessageGenerator(dgh.getArgs()));
		}

		reloadFramework();
	}

    private void showWhatHappens()
    {
		JOptionPane.showMessageDialog(frame,
				    dgh.getGameExplanationMessage(),
				    "What's happening?",
				    JOptionPane.INFORMATION_MESSAGE);
	}

    private void showHelp()
    {
		helpFrame.setVisible(true);
	}

    private void showAbout()
    {
		String about =  "Implementation of Argument Based Discussion\n\n" +
						"Authors:\n" +
						"\tPatrizio Barbini (patrizio.barbini@uni.lu),\n" +
						"\tYining Wu (yining.wu@uni.lu)\n\n";

		JOptionPane.showMessageDialog(frame,
				    about,
				    "About this program...",
				    JOptionPane.INFORMATION_MESSAGE);
	}

    private void showExtensions()
    {
		if (!extensionsFrame.isVisible())
		{
			extensionsFrame.setVisible(true);
		}
	}

    private void updateArgList(String[] newList)
    {
		argList.setListData(newList);
		argList.setSelectedIndex(0);
	}

    public void output(String s, int player)
    {
		String message = "";

		if (player == DiscussionGameOutput.APPLICATION)
			message = "<P style=\"background-color:#eeeeee;font-weight:bold\">Computer says:</P><P style=\"font-family:verdana\">" + s + "</P>";
		else if (player == DiscussionGameOutput.USER)
			message = "<P style=\"background-color:#feffb8;font-weight:bold\">User says:</P><P style=\"font-family:verdana\">" + s + "</P>";
		else if (player == DiscussionGameOutput.SYSTEM)
			message = "<P style=\"background-color:#ff0000;font-weight:bold\">" + s + "</P>";

		output += message;

		outputView.setText(output);
	}
}
