import argumentation.*;

import java.io.File;
import java.io.StringReader;
import java.io.IOException;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;


import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.ListSelectionModel;
//import javax.swing.event.ListSelectionListener;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.SpringLayout;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.JEditorPane;
import javax.swing.JTabbedPane;

import java.awt.*;
import java.awt.event.*;
import javax.swing.KeyStroke;

import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JFrame;

/**
The <CODE>ExtensionLayout</CODE> is the component which displays the extensions of an argumentation framework.
*/
public class ExtensionLayout extends JFrame implements ActionListener
{
	private DiscussionGameHandler dgh;
	private ArgumentationFramework em;
	private JTabbedPane tabbedPane;
	private JTextArea groundedExtList;
	private JTextArea[] extList;
	private Extension groundedExtension;
	private Extension[][] extensions;
	private String[] arguments;

	private JLabel[] statusLabel;
	private int[] currentSel;

	/**
	Construct a new <CODE>ExtensionLayout</CODE> for the given <CODE>DiscussionGameHandler</CODE>.
	@param title the title for this frame
	@param d the <CODE>DiscussionGameHandler</CODE> of the application
	*/
	public ExtensionLayout(String title, DiscussionGameHandler d)
	{
		super(title);
		dgh = d;
		add(createExtContentPanel());
	}

	/*
	Create the panel
	*/
	private JPanel createExtContentPanel()
	{
		extList = new JTextArea[3];
		extensions = new Extension[3][];
		statusLabel = new JLabel[3];

		//Create the content-pane-to-be.
		JPanel contentPane = new JPanel(new BorderLayout());
		contentPane.setOpaque(true);

		//Create the tabbed pane
		tabbedPane = new JTabbedPane();

		JPanel groundedPanel = new JPanel(new BorderLayout());
		groundedExtList = new JTextArea();
		groundedExtList.setEditable(false);
		JScrollPane grScrollPane = new JScrollPane(groundedExtList);
		groundedPanel.add(grScrollPane, BorderLayout.CENTER);

		JPanel preferredPanel = new JPanel(new BorderLayout());
		extList[0] = new JTextArea();
		extList[0].setEditable(false);
		JScrollPane prScrollPane = new JScrollPane(extList[0]);
		preferredPanel.add(prScrollPane, BorderLayout.CENTER);
		statusLabel[0] = new JLabel("");
		preferredPanel.add(statusLabel[0], BorderLayout.NORTH);
		preferredPanel.add(createButtonPanel(), BorderLayout.SOUTH);

		JPanel sStablePanel = new JPanel(new BorderLayout());
		extList[1] = new JTextArea();
		extList[1].setEditable(false);
		JScrollPane ssScrollPane = new JScrollPane(extList[1]);
		sStablePanel.add(ssScrollPane, BorderLayout.CENTER);
		statusLabel[1] = new JLabel("");
		sStablePanel.add(statusLabel[1], BorderLayout.NORTH);
		sStablePanel.add(createButtonPanel(), BorderLayout.SOUTH);

		JPanel stablePanel = new JPanel(new BorderLayout());
		extList[2] = new JTextArea();
		extList[2].setEditable(false);
		JScrollPane stScrollPane = new JScrollPane(extList[2]);
		stablePanel.add(stScrollPane, BorderLayout.CENTER);
		statusLabel[2] = new JLabel("");
		stablePanel.add(statusLabel[2], BorderLayout.NORTH);
		stablePanel.add(createButtonPanel(), BorderLayout.SOUTH);

		tabbedPane.addTab("Grounded", groundedPanel);
		tabbedPane.addTab("Preferred", preferredPanel);
		tabbedPane.addTab("Semi-stable", sStablePanel);
		tabbedPane.addTab("Stable", stablePanel);
		tabbedPane.setMnemonicAt(0, KeyEvent.VK_1);
		tabbedPane.setMnemonicAt(1, KeyEvent.VK_2);
		tabbedPane.setMnemonicAt(2, KeyEvent.VK_3);
		tabbedPane.setMnemonicAt(3, KeyEvent.VK_4);

		//Add the tabbed pane to the panel.
		contentPane.add(tabbedPane);

		tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

		return contentPane;
	}

	/*
	Creates the button panel for the preferrd-semistable-stable extensions' panel
	*/
	private JPanel createButtonPanel()
	{
		//Create the button panel
		JPanel buttonPanel = new JPanel(new GridLayout(1, 2));
		JButton buttonLeft = new JButton("Previous");
		JButton buttonRight = new JButton("Next");
		buttonLeft.setActionCommand("PREV");
		buttonRight.setActionCommand("NEXT");
		buttonLeft.addActionListener(this);
		buttonRight.addActionListener(this);
		buttonPanel.add(buttonLeft);
		buttonPanel.add(buttonRight);

		return buttonPanel;
	}

	/**
	Update the content of this <CODE>ExtensionLayout</CODE>.
	*/
	public void updateContent()
	{
		em = dgh.getExtensionManager();

		arguments = dgh.getMessageGenerator().getArgs();

		Extension groundedExt = em.getGroundedExtension();

		extensions = new Extension[3][];
		extensions[0] = em.getPreferredExtensions();
		extensions[1] = em.getSemiStableExtensions();
		extensions[2] = em.getStableExtensions();

		statusLabel[0].setText("Found " + extensions[0].length + " preferred extensions.");
		statusLabel[1].setText("Found " + extensions[1].length + " semi-stable extensions.");
		statusLabel[2].setText("Found " + extensions[2].length + " stable extensions.");

		currentSel = new int[3];

		groundedExtList.setText(extensionToText(groundedExt));
		extList[0].setText(extensionToText(extensions[1][currentSel[0]]));
		extList[1].setText(extensionToText(extensions[1][currentSel[1]]));


		if (extensions[2].length == 0)	//if there are no stable extensions, disable the panel
			tabbedPane.setEnabledAt(3, false);
		else						//re-enable it after a previous disabling, and load selections
		{
			tabbedPane.setEnabledAt(3, true);
			extList[2].setText(extensionToText(extensions[2][currentSel[2]]));
		}
	}

	/**
	This method listens the action on this <CODE>ExtensionLayout</CODE>'s components.
	@param e the event to be listened
	*/
    public void actionPerformed(ActionEvent e)
	{
		JButton source = (JButton)e.getSource();

		int listInd = tabbedPane.getSelectedIndex() - 1; //- 1 because the first is actually the grounded, which doesn't have buttons

        if (source.getActionCommand().equals("NEXT"))
        	extList[listInd].setText(extensionToText(extensions[listInd][nextSelection(listInd)]));
		else
			extList[listInd].setText(extensionToText(extensions[listInd][previousSelection(listInd)]));
	}

	/*
	Compute the index of the next extension to be displayed in the currently displayed extension panel.
	*/
	private int nextSelection(int ext)
	{
		int extNum = extensions[ext].length;
		currentSel[ext] = ( currentSel[ext] + 1 ) % extNum;

		return currentSel[ext];
	}

	/*
	Compute the index of the previous extension to be displayed in the currently displayed extension panel.
	*/
	private int previousSelection(int ext)
	{
		int extNum = extensions[ext].length;
		currentSel[ext] = (( currentSel[ext] - 1 ) + extNum ) % extNum;

		return currentSel[ext];
	}

	/*
	Convert the boolean array of an Extension to indices on the argument's list.
	*/
	private String extensionToText(Extension e)
	{
		String ext = "";
		for (int i = 0; i < e.args.length; i++)
			if (e.args[i])
				ext += arguments[i] + "\n";

		return ext;
	}
}