package argumentation;

/**
The Preferred Game etc etc
*/
public class PreferredGamePlayer implements ArgumentationGamePlayer
{
	private static final int PROPONENT = 0;
	private static final int OPPONENT = 1;

	private int[][] playerChoices;
	private boolean started;
	//TODO: optimize next() and hasNext()
	private Node game;

	private int winner;

	boolean[][] def;


	/**
	Build etc
	*/
	public PreferredGamePlayer(boolean[][] def)
	{
		this.def = def;
	}

	/**
	*/
	public void init()
	{
		playerChoices = new int[2][];
		playerChoices[PROPONENT] = new int[def.length];
		playerChoices[OPPONENT] = new int[def.length];

		started = false;
	}

	public int distance()
	{
		if (started)
			return game.distance;
		return -1;
	}

	public boolean[] next()
	{
		boolean[] next = new boolean[game.next.length];
		for (int i = 0; i < game.next.length; i++)
			if (game.next[i] != null)
				next[i] = true;
		return next;
	}

	public boolean hasNext()
	{
		for (int i = 0; i < game.next.length; i++)
			if (game.next[i] != null)
				return true;
		return false;
	}


	public int argue(int argument)
	{
		if (argument < 0 || argument >= def.length)
			throw new IllegalArgumentException("Not a correct argument for the current framework.");

		//0. if the game hasn't been started yet, build the game and start the game
		if (!started)
		{
			game = buildGame(argument, new boolean[def.length], new boolean[def.length], PROPONENT);

			started = true;

			if (game.winner == PROPONENT)
				return PLAY_PROPONENT;
		}
		else //1. check argument for move correctness
		{
			if (game.next[argument] != null)
				game = game.next[argument];
			else
				return WRONG_MOVE;
		}

		//2. choose next move
		int choice = -1;
		for (int i = 0; i < game.next.length; i++)
		{
			if (game.next[i] != null && game.next[i].winner == game.winner)
				if (choice == -1 || game.next[i].distance < game.next[choice].distance)
					choice = i;
		}

		game = game.next[choice];

		return choice;
	}


	/*
	Builds the game tree for the discussion, starting on the argument
	*/
	private Node buildGame(int argument, boolean[] propMoves, boolean[] oppMoves, int player)
	{
		//System.out.println("PLAYING FOR " + player);
		Node currentNode = new Node();
		currentNode.winner = player;

		if (player == OPPONENT && propMoves[argument])
		{
			currentNode.distance = 0;
			return currentNode;
		}

		int[] distance = {Integer.MAX_VALUE, Integer.MAX_VALUE};	//distance[PROPONENT] = distance to proponent's victory
																	//distance[OPPONENT] = distance to opponent's victory

		boolean[] newOppMoves;		//We update opponent's done moves
		boolean[] newPropMoves;		//...and proponent's done moves

		if (player == OPPONENT)
		{
			newOppMoves = update(oppMoves, argument);
			newPropMoves = propMoves;
		}
		else			//The player is the proponent
		{
			newPropMoves = update(propMoves, argument);
			newOppMoves = oppMoves;
		}


		//NOW, EVALUATE MOVES FOR NEXT PLAYER

		boolean hasDefeater = false;

		for (int i = 0; i < def.length; i++)
		{
			if (!(oppMoves[i]) &&
			     	( def[i][argument] ||
			     		( player == PROPONENT && defeatsSome(i, propMoves) )
			     	)
			   )
			{
				//System.out.println("Choosing for " + nextPlayer(player) + ", evaluating " + i);
				hasDefeater = true;

				currentNode.next[i] = buildGame(i, newPropMoves, newOppMoves, nextPlayer(player));

				if (distance[currentNode.next[i].winner] > currentNode.next[i].distance + 1)	//update the distance for next node's winner.
					distance[currentNode.next[i].winner] = currentNode.next[i].distance + 1;

				if (currentNode.next[i].winner == nextPlayer(player))		//if the next player can win, he will.
					currentNode.winner = nextPlayer(player);
			}
		}

		if (player == OPPONENT && !hasDefeater)		//The proponent cannot move anymore...
			currentNode.distance = 0;
		else
			currentNode.distance = distance[currentNode.winner];

		return currentNode;
	}

	/*
	Switches the players.
	*/
	private int nextPlayer(int player)
	{
		if (player == OPPONENT)
			return PROPONENT;
		return OPPONENT;
	}

	private boolean defeatsSome(int a, boolean[] arguments)
	{
		for (int i = 0; i < arguments.length; i++)
			if (arguments[i] && def[a][i]) return true;
		return false;
	}

	/*
	Add a new move to the proponent's already done moves.
	*/
	private boolean[] update(boolean[] propMoves, int argument)
	{
		boolean[] newMoves = new boolean[propMoves.length];

		for (int i = 0; i < propMoves.length; i++)
			newMoves[i] = propMoves[i];

		newMoves[argument] = true;

		return newMoves;
	}

	/*
	Set A-
	Returns the set of defeaters of the argument given as input.
	B is in defeaters(A) iff B is in A-
	*/
	private boolean[] getDefeaters(int a)
	{
		boolean[] defeaters = new boolean[def.length];
		for (int i = 0; i < defeaters.length; i++)
			if (def[i][a])
				defeaters[i] = true;
		return defeaters;
	}


	/*
	A node of the tree storing the discussion outcome.
	*/
	private class Node
	{
		public int winner;		//the winner of the discussion starting from this node;
								//the winner is this node's player if the winner for at
								//least one of the next nodes it this node's player; the
								//winner is the other player.
		public int distance;	//the number of moves until the victory
		public Node[] next;		//next arguments in the discussion

		public Node()
		{
			next = new Node[def.length];
		}
	}
}


/*
	public void argue(int argument)
	{
		Node game = buildGame(argument, new boolean[args.length], PROPONENT);

		if (game.winner == PROPONENT)
			System.out.println("I'll argue " + args[argument]);
		else if (game.winner == OPPONENT)
			System.out.println("I'll argue against " + args[argument]);

		System.out.println("I'll win in " + game.distance + " moves! :-P");

		//second step - game playing

		System.out.println("P:" + args[argument]);

		int turn = OPPONENT;
		Node currentArgument = game;

		while (currentArgument.distance > 0)
		{
			if (turn == PROPONENT)
				System.out.print("P:");
			else
				System.out.print("O:");

			if (turn == game.winner)
			{
				int choice = -1;

				for (int i = 0; i < currentArgument.next.length; i++)
				{
					if (currentArgument.next[i] != null && currentArgument.next[i].winner == game.winner)
						if (choice == -1 || currentArgument.next[i].distance < currentArgument.next[choice].distance)
							choice = i;
				}

				currentArgument = currentArgument.next[choice];

				System.out.println(args[choice]);

				turn = nextPlayer(turn);
			}
			else
			{
				String newArg = input.nextLine();
				argument = findIndex(newArg);

				if (currentArgument.next[argument] != null)
				{
					currentArgument = currentArgument.next[argument];
					turn = nextPlayer(turn);
				}
				else
					System.out.println("Not a valid argument!");
			}
		}
	}
*/